module LiveQA
  ##
  # == LiveQA \Config
  #
  # Represent the LiveQA configuration for the API
  class Config

    DEFAULT_OBFUSCATED_FIELDS = %w[
      password
      password_confirmation
      secret
      secret_token
      authenticity_token
      token
      api_key
      access_token
      credit_card_number
      cvv
      ccv
    ].freeze

    ##
    # @return [String] API account token
    attr_accessor :account_token

    ##
    # @return [String] API space name
    attr_accessor :space_name

    ##
    # @return [String] API environment name
    attr_accessor :environment_name

    ##
    # @return [String] API host
    attr_accessor :api_host

    ##
    # @return [String] API version
    attr_accessor :api_version

    ##
    # @return [String] proxy url
    attr_accessor :proxy_url

    ##
    # @return [Boolean] http secure
    attr_accessor :http_secure

    ##
    # @return [Boolean] service is enable
    attr_accessor :enabled

    ##
    # @return [Array[String]] fields to obfuscate
    attr_accessor :obfuscated_fields

    ##
    # @return [Boolean] send
    attr_accessor :async

    ##
    # @return [Hash] custom object properties
    attr_accessor :custom_object_properties

    ##
    # @return [Hash] metadata to be attach to the payload
    attr_accessor :metadata

    ##
    # @return [Boolean] enable the logger
    attr_accessor :log

    ##
    # @return [Logger] logger
    attr_accessor :logger

    ##
    # @param [Hash{Symbol=>Object}]
    # Initialize and validate the configuration
    # rubocop:disable Metrics/CyclomaticComplexity
    # rubocop:disable Metrics/PerceivedComplexity
    def initialize(options = {})
      self.account_token = options[:account_token]
      self.space_name = options[:space_name]
      self.environment_name = options[:environment_name]
      self.api_host = options[:api_host] || 'api.liveqa.io'
      self.api_version = options[:api_version] || 'v1'
      self.proxy_url = options[:proxy_url]
      self.http_secure = options[:http_secure] || true
      self.enabled = options[:enabled] || true
      self.obfuscated_fields = options[:obfuscated_fields] || []
      self.async = options[:async] || true
      self.custom_object_properties = options[:custom_object_properties] || {}
      self.metadata = options[:metadata]
      self.log = options[:log] || true
      self.logger = FormatedLogger.build(options[:logger])
    end
    # rubocop:enable Metrics/CyclomaticComplexity
    # rubocop:enable Metrics/PerceivedComplexity

    ##
    # validate the configuration
    # Raise when configuration are not valid
    # @return [Boolean] true
    def valid!
      format!

      %i[account_token space_name environment_name api_host api_version].each do |field|
        validate_presence(field)
      end

      true
    end

    private

    ##
    # Format configuration fields
    #
    # * Set obfuscated_fields to string
    #
    def format!
      self.obfuscated_fields = (obfuscated_fields.map(&:to_s) + DEFAULT_OBFUSCATED_FIELDS).uniq
    end

    def validate_presence(field)
      raise LiveQA::ConfigurationError, "#{field} can't be blank" if send(field).nil? || send(field).empty?
    end

  end
end
